/************************************************************************/
/*                                                                      */
/*   File name :  libmk5tok5.h                                          */
/*                                                                      */
/*      Version:   2007-07-18                                           */
/*                                                                      */
/*   Copyright (c) 2003-2007 T.Kondo/NICT(CRL) All Right Reserved       */
/*                                                                      */
/************************************************************************/
/*                                   
 library for Mark5 to K5 conversion program
     2003-03-13
     2005-06-17

2007-07-18
     the number of characters of k5 file name and Mark5 file name was extended 
	 from 80 to 256

2006-09-26
     parameter (vcntable : video comberter # table)) was added to 
     arguments of 'mk5_dat_anal'

2005-09-17
     use 'LL' like 0xFFFFFFFFFFFFFFFFLL to express int64_t constant 
     to avoid warning message appearing in the latest version of gcc.

2005-06-17
      divided into libmk5tok5.h and libmk5tok5.c
      
2004-11-14
     "get_bit_track" was update to correct bit position order in case of adbit=2
      (it was wrong)

2004-11-14
     functions for K5 to Mark5 conversion was included

2004-09-25
     "mk5_dat_anal" was updated to display 1st data block just after 
     header block

2004-09-24
    debug in "conv2k5" for VLBA mode

2004-09-13
    "make_mk5tok5_info_file" : head stack information was added. Track 
    information in an information file created was modified so as to 
    include head stack number.  TT  -->  H-TT  where TT is track 2 digit 
    number and H is head stack number (1,2,3,4).
    Head stack information was added in mk5tok5_info structual variable
    "mk5tok5_info_setup" : head stack information subtraction was added

2004-09-06
   "make_mk5tok5_info_file" was renamed to "make_mk5tok5_info_file_old"
   "make_mk5tok5_info_file_r" was renamed to "make_mk5tok5_info_file"

2004-09-03
   mk5_data type structure :  new members were added

2004-09-02
  "mk5tok5_info_setup" : serious bug was fixed.
      track information was wrong when adbit=1 and fanout =2 or more

2004-06-29
   "make_mk5tok5_info_file_np", "make_mk5tok5_info_file_r": default tape pass
   was added.
   used "mk5_dat_anal" was renamed to "mk5_dat_anal_old".
   "mk5_dat_anal_np_r" was renamed to "mk5_dat_anal" and modified a bit


2004-04-07
   In fast_atamadashi_np, fast_atamadashi
   nkai was modified to keep enough margin as follows

   nkai=(int)((dbits-(double)ndata)/(double)nbuf)-20;  //2004-04-07 for get more margin

2003-10-28
   make_mk5tok4_info_file_r was developed to handlr both parity and non-parity 
   format data

2003-10-12
   mk5tok5_info structure # of tracks was increased from 32 to 64
   make_mk5tok5_info_file and make_mk5tok5_info_file_np were debugged and
   modified to allow 64 track mode
   samplig frequency of faster than 16MHz (i.e., 32 MHz, 64MHz,.. up to 2048MHz)
   was added in ip_hd_make_ini

2003-08-01
   make_mk5tok5_info_file_np was updated to make non-interactive operation
   possible
2003-07-16
   debug get_bit_track
   debug infast_atamadashi_np
2003-07-12
   sync_detect2new was modified
   mk5_dat_anal_np was modified
   make_mk5tok5_info_file_np was modified
   ntrack is added in the member of "typedef struct mk5tok5_info" 
   mk5tok5_info_setup was modified
   read_1frame_np was modified
   read_ndata_np is used instead of read_ndatas
*/
#ifndef _LIBMK5TOK5_
#define _LIBMK5TOK5_

#include "libskdvex.h"

#ifndef PIRA
#define PIRA  1.745329252E-2    // PI/180  degree to radian
#endif
#ifndef PIDE
#define PIDE  57.29577951       // 180/PI radian to degree
#endif
#ifndef TWOPI
#define TWOPI 6.283185307       // 2PI
#endif
#ifndef PI
#define PI    3.141592653
#endif


#ifdef WIN32
#ifndef _FORWIN32SET_
#define _FORWIN32SET_
typedef unsigned int    u_int32_t;
typedef _int64 int64_t;
#define popen(_Z,_X) _popen(_Z,_X)
#endif
#endif

typedef struct k5m5runp     // run parameters for k5 to mark5 conversion program
{
    int ptype;   // run parameter type  =1  mode1,  =2 mode 2
    int channel;
    int group;
    int soffset;
    int period;
    int reverse;
    int makemode;
    int scan;    // scan# for get MODE from VEX file (default=1) 
    int odd;    // =1 4byte integer offset mode in case of 64 track
    int monit;   // =1  inofo_file monitor ON,  =0 no monitoring
    int yyyy, mm, dd, ddd;  // observation date year month,day of month, day of year
    int mjd;   //  MJD
    int mjd3;   // lower 3 digit of MJD
    int k5num;
    char mk5name[256];  // 2007-07-17  80->256
    char k5name[4][256];  // 2007-07-17  80->256
    char outdir[256];     // 2007-07-17  80->256
    char info_file[256]; // 2007-07-17  80->256
    char vex_file[256]; // 2007-07-17  80->256
    char sid[4];
    char dflt_pass[4];
    // for mode2 only run parameres are as follows
    int numch;
    int ntrack;
    int adbit;
    int fanout;
    int parity;
    int nrzm;
    int vlba;
    int ibit[64];
    int tot_track;
    int nbits;
    double fmhz;
} k5m5runp_t;


typedef struct m5k5runp     // run parameters for mark5 to k5 conversion program
{
    int ptype;   // run parameter type  =1  mode1,  =2 mode 2
    int channel;
    int group;
    int soffset;
    int period;
    int reverse;
    int makemode;
    int scan;    // scan# for get MODE from VEX file (default=1) 
    int odd;    // =1 4byte integer offset mode in case of 64 track
    int monit;   // =1  inofo_file monitor ON,  =0 no monitoring
    char mk5name[256];   // 2007-07-17  80->256
    char k5name[256]; // 2007-07-17  80->256
    char k5name_given[256]; // 2007-07-17  80->256
    char outdir[256];      // 2007-07-17  80->256
    char info_file[256]; // 2007-07-17  80->256
    char vex_file[256]; // 2007-07-17  80->256
    char sid[4];
    char dflt_pass[4];
    // for mode2 only run parameres are as follows
    int numch;
    int ntrack;
    int adbit;
    int fanout;
    int parity;
    int nrzm;
    int vlba;
    int ibit[32];
    int tot_track;
    double fmhz;
} m5k5runp_t;

typedef struct mk5tok5_info
{
    int m5tok5mode;  // =1: this parameter is for Mark5 to K5 conversion
                     // =0: this parameter is for K5 to Mark5 conversion
    int nch;         // valid channel number in signtrack, hs_sign,
                     //   magtrack, hs_mag table  (usually 14 or 16)
    int adbit;       // AD resolution
    int fanout;      // fouout 
    double samplrate;     // Sampling frequency (Hz)
    int signtrack[16][4]; // channel vs sign track table (fan out max = 4)
    int hs_sign[16][4];   // head stack info for sign track 2004-09-13
    int magtrack[16][4];  // channel vs mag track table (fan out max = 4)
    int hs_mag[16][4];   // head stack info for mag track 2004-09-13
    int ch_ch[16];      // actual Channel # table 2004-12-25
    int ch_bbc[16];      // channel vs bbc# 2004-12-25
    int ch_lsb[16]; // channel vs sideband  0:USB  1:LSB  2004-12-25
    int track_channel[64]; // track versus channel table  2003-10-10
    int bit_track[64];  // bit position versus track table  2003-10-10
    int bit_hs[64];  // bit position versus headstack number table  2004-9-13
    int bit_valid[64]; // valid track (bit position) table  0: invalid  1: valid
    int group_channel[4][4];    // group versus channel table
    int ngroup;    // valid number of groups usually 4
    int ntrack;    // bit# per word  = # of Mark5 tracks  2003-07-12
    int parity;    // =1: with parity  =0; non-parity   2003-10-28
    int nrzm;      // =1: NRZM         =0; NRZL         2003-10-28
    int vlba;      // =1: VLBA format  =0; Mark-IV format 2003-10-28
    int bitsframe; // number of bits per frame (without parity)
                   // =20000: Mark-IV  =20160: VLBA
} mk5tok5_info_t;


typedef struct gr_info
{
    int num_tracks_per_ch; // number of tracks per channel
    int tot_track;         // total number of tracks
    int numch;             // number of channel
    int group;             // group # to pickup
    int channel;           // channel # to pickup, if this is not zero,
                           // group parameter is omitted   
    int reverse;           // reversal flag
    int bit_trk[64];  // track table with K5 conversion channel order  2003-10-10
	//int validtrk[64];
    //int trktable[64];  // bit position vs tarck table
    //int h_stack[64];  // bit position head stack table  2004-9-13

    int ch_table[4];    // channel table
} gr_info_t;

typedef struct mk5_aux
{
	int h_pos1;    // headstack 1 position in microns
	int h_pos2;    // headstack 2 position in microns
	int h_stack;    // headstack number 1-4
    int trk;        // track number 
    int foid;     // fan-out sub-channel number   0,1,2,3
    int sormid;     // Sign or Magnitude bit ID   0:Sign bit  1:Magnitude bit
    int uorlid;     //USB or LSB ID   0:USB  1:LSB
	int vcn;         // Video converter number minus 1 (0-15)
	int adid;         // AD id
    int eflag[8];    // Formatter error flag
    int id;         // system ID
} mk5_aux_t;


typedef struct mk5_data
{
	FILE *fin;
	//int idata[20000];    // 1 frame data
	int idata[20160];    // 1 frame data (to allow VLBA format)
	//int idata[22680];    // 1 frame data (to allow VLBA format with parity bit)
	int ndata;           // valid data number in idata
	double dtime;        // time at begin of frame   sec in day
	int itime[6];        // time at begin of frame y,ddd,hh,mm,ss,msec
	int header[5];       // header of frame 
	int bitpos;          // bit position in the word corresponding to track position
	int ko;              // for decode NRZL
	double fsampl;       // supposed track sampling frequency 
                         // In case of fanout=2, fsampl is half of data sampling rate
    mk5_aux_t aux;       // AUX field structured data 
    /*  added on 2004-09-03 */
    int data_mode;       // Mk5 data mode
                         //  1: NRZM with Parity     (NRZM->NRZL necessary)
                         //             initial Mark-V
                         //  2: NRZM without parity  (NRZM->NRZL necessary)
                         //             so far never seen
                         //  3: NRZL with parity
                         //             so far never seen
                         //  4: NRZL without parity  
                         //             recent Mark-V
    int parity;    // =1: with parity  =0; non-parity   2003-10-28
    int nrzm;      // =1: NRZM         =0; NRZL         2003-10-28
    int ntrack;    // bit# per word  = # of Mark5 tracks  
    int vlba;      // =1: VLBA format  =0; Mark-IV format 
    int bitsframe; // number of bits per frame (without parity)
                   // =20000: Mark-IV  =20160: VLBA
} mk5_data_t;


typedef struct mk5_data_r     
{
    //          2004-09-01
    //      track data structure variable 
	FILE *fin;
	//int idata[20000];    // 1 frame data
	unsigned int idata[20160];    // Max 32 track X 1 frame data (to allow VLBA format)
                         // in bit image
                         // 20160=32*((20160/8)/4)
    int xmax;            //  = 20160
	int ndata[32];    // valid data number (bit # in each track )
	unsigned int workd[22528];    // Max 32 track X 1 frame data with parity  (to allow VLBA format)
    int wmax;         // =22528;            / 
                         //  n-th bit in a frame for k-th track is 
                        //   (k-1)*xmax+n th bit in idata
                        // in bit image
    int wp[32];       // next writing bit position in workd by tarck

	double dtime[32];        // time at begin of frame   sec in day
	int itime[32*6];        // time at begin of frame y,ddd,hh,mm,ss,msec
	int header[32*5];    // header of frame Max 32 track
	int bitpos[32];   // bit position table in the word corresponding to track position
	int ko[32];          // for decode NRZL
	double fsampl[32];       // supposed track sampling frequency
    int tot_track;       // 
    mk5_aux_t aux[32];   // AUX field structured data 
    mk5tok5_info_t mk5info;  // Mark5->K5 conversion information

} mk5_data_r_t;


typedef struct mk5_read     
{
    //          2004-09-13
    //      general mark 5 data read structure variable 
	FILE *fin;
	unsigned int *ibuf;  // pointer data read buffer
	int ibufsize;        // ibuf size
    unsigned short *sbuf;
    unsigned char *cbuf;
    int64_t *lbuf;  
	int nbuf;            // number of elements to read
	int ntrack;          // number of tracks (bits) per word
	int nrzm;            // 0: NRZL   1:NRZM
	int parity;          // 0: non parity  1: with parity
	int itime[6];        // time at begin of frame y,ddd,hh,mm,ss,msec
	double fsampl;       // supposed track sampling frequency 
                         // In case of fanout=2, fsampl is half of data sampling rate
	unsigned int kold;   // data to decode NRZM data
    int64_t lkold;       // data to decode NRZM data in case of 64 tracks mode
                              // not used yet
	int iread;           // # of data actually read 
	int ipos;            // next data pointer in ibuf 
    int *header;         // pointer to header data save
    int *head180;        // pointer to header bit data
    int nbithead;        // # of bits in header  
                         //   180 for parity=1   160 for parity=0
    int nbitword;        // # of bits in a header word
                         //   36 for parity=1   32 for parity=0
    int nbitbyte;        // # of bits in a byte
                         //   9 for parity=1   8 for parity=0
    int tot_track;       // # of tracks to process simultaneously
    int bitsframe; // number of bits per frame (without parity)
                   // =20000: Mark-IV  =20160: VLBA
    int vlba;     // =0: Mark-IV,  =1: VLBA mode
    int bit_trk[64];  // track table with K5 conversion channel order 

} mk5_read_t;

typedef struct k5_write     
{
    //          2004-09-13
    //      general mark 5 data read structure variable 
	FILE *fo;
	int *obuf;  // pointer data read buffer
	int nwrite;        // obuf size
} k5_write_t;


typedef struct ip_vlbi_file_old   // type name change 
{
	unsigned char bit32[4];
	unsigned char bit64[8];
	unsigned char *ibuf;
    int *idata;
	int sfreq;
	int icnt;   //  current unit number in a second data 
	int numb;   //  # of bytes in unit # of samples
	int usampl;  // unit sample number
	int kpos;
	int imax;    //  # of units in a second data
	char* fname; //I[vĂt@C
	double fsample;  //TvOg(Hz)
	double tsample; // sampling period
	double unitp;   // usampl*tsampl
	int   adbit;  // ADrbg
	long int seconds; // 00h00m00šoߕb
	int hh, mm, ss;
	int spatn; // sync pattern    
	int syncok; // =1 for 32bit all 1 detection // not used
	int format; // =0 for old format , =1 for new format
	FILE *stream;
	fpos_t pos;
	int fh;
	int ch;    
	int numch;
	int ierr;
	double dtime;  // time at begin of unit
	double dtimest;  // time at begin of file data
	unsigned int bytesread;
    int vflag, vflag_old;  // byte slip flag  added on 2003-02-14
                           //  if header sync detection is suceed
                           //     set to 1.  else 0
                           //   1: sync detection success 
                           //   0: sync detection error (may be byte slip)
                           //   vflag_old is the previous status
    long int dtime_vflag, dtime_vflag_old; // time at vflag decision made
} ip_vlbi_file_old_t;


typedef struct ipboard {
	int adbit;         // A/Drbg (1,2,4,8)
	int numch;         // ͂@(1,4)
	int sfreq;         // TvOg(kHz)(40,100,200,500,1000,2000,4000,8000,16000)
    long int seconds;    // Ǒoߕb
    unsigned char hd[8]; // ho{[hwb_[ 
    long int nbsec;       // Pbf[^̃oCgiwb_[܂ށj 
    long int bpos;       // ݊JnoCgʒuiOX^[gj
} IPBOARD_T;

typedef struct m5head {    // Mark5 headder type including conversion info
    int ihead[180];
    short int shead[180];
    int64_t lhead[180];
    unsigned int io;
    int64_t lo;
	int adbit;         // A/D quantization number (1,2,4,8)
	int numch;         // ͂@
	int sfreq;         // TvOg(kHz)(40,100,200,500,1000,2000,4000,8000,16000,32000)
    int mjd;         // lowest 3 digits of MJD
    int ddd;         // day of year
    int yyyy;        // 4 digit year
    int seconds;    // integer seconds
    int ifsec;    // fractional second in 0.1 msec unit
    int ifsecframe; //  frame period in 0.1msec unit
    unsigned char hd[8]; // ho{[hwb_[ 
    long int nbsec;       // Pbf[^̃oCgiwb_[܂ށj 
    long int bpos;       // ݊JnoCgʒuiOX^[gj
    int nbits;   // bits per word 16,32,64 (# of tracks)
    int fanout;      // fanout
    int chinfo[64];      // channel# info   0 no data  1, 2, 3, .....
    int adinfo[64];      // A/D bit info   -1 no data  0: sign bit 1: amp bit
    int faninfo[64];     // fanout # info  0 no data  1, 2, ...  fanout#
    int headstackinfo[64]; // head stack# info  0 no data  1, 2, ...  
    int trkinfo[64];     // track# info  0 no data  1, 2, ...  
    int lsbinfo[64]; // sideband (LSB) info  1: LSB  0:USB  
    int vcinfo[64]; // Video converter # 0 no data 1,2,...  
    int parity;    // =1: with parity  =0; non-parity   
    int nrzm;      // =1: NRZM         =0; NRZL         
    int vlba;      // =1: VLBA format  =0; Mark-IV format 
    int bitsframe; // number of bits per frame (without parity)
                   // =20000: Mark-IV  =20160: VLBA
} m5head_t;

void headerchkK5(unsigned char* bit64, int* iok, int* spatn, int* smplbit,
			   int* sfreq, int* numch, long int* seconds, int* hh,
			   int* mm, int* ss);
int chk_K5_header(ip_vlbi_file_old_t *S);
int fwd_K5_Nsec(unsigned char* ibuf, int nsec, int numb, int imax,ip_vlbi_file_old_t *dat);
int crcc(unsigned char* idata, int len, int mask, int cycl);
int crc_12(unsigned char* idata, int len);
int crc_16(unsigned char* idata, int len);
int parity(unsigned char c);
void m5_hd_synctime_make(m5head_t *hd);
void m5_hd_synctime_make64(m5head_t *hd);
void m5_hd_make(m5head_t *hd);
void m5_hd_make64(m5head_t *hd);
void m5_hd_make_IV(m5head_t *hd);
void ip_hd_make_ini(IPBOARD_T *ipb);
void ip_hd_make(IPBOARD_T *ipb);
int nrzm2nrzl(int nrzmold, int nrzmnow);
void nrzm2nrzl_a(int *nrzm, int *nrzl, int nkai);
int rvsint(int i4);
int rvsint_nbit(int i4, int nbit);
void print_intbit(int i4);
int conv4byte2int(unsigned char *b4);
int conv2int(int *idat36, int numch);
int conv2intn(int *idat36, int numch);
void conv2sint(int *tdat, short int *sint);
void conv2time(int *idat, int *itime);
void conv2time_vlba(int *idat, int *itime);
void conv2time2(int *tdat, int *itime);
void conv2time2n(int *tdat, int *itime);
void conv2time_full(int *tdat, int *itime, int parity, int vlba);
int bcd4digit2int(int bcd4digit);
int bcd2digit2int(int bcd2digit);
void parseMK4aux(int *header, mk5_aux_t *aux);
void get_trk_from_header(int *header, int *trk, int *adid);
void get_trk_from_header_a(int *header, int *trk, int *adid, int vlba);
void parseMK5aux(int *header, int vlba, mk5_aux_t *mk5aux);
void header2int(int *head180, int *head5);
int read_ndata(FILE *fin, int bitpos, int ndata, int *idata, int *ko);
int read_ndatas(FILE *fin, int bitpos, int ndata, int *idata, int *ko);
int read_ndata_np(FILE *fin, int bitpos, int ndata, int *idata, int *ko, int ntrack);
int read_ndata_r(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko, int ntrack, int nrzm);
int read_ndata8_r(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko, int nrzm);
int read_ndata16_r(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko, int nrzm);
int read_ndata32_r(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko, int nrzm);
int read_ndata64_r(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko, int nrzm);
int read_ndata8_m(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko);
int read_ndata16_m(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko);
int read_ndata32_m(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko);
int read_ndata64_m(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko);
int read_ndata8_l(FILE *fin, int bitpos, int ndata, int *idata);
int read_ndata16_l(FILE *fin, int bitpos, int ndata, int *idata);
int read_ndata32_l(FILE *fin, int bitpos, int ndata, int *idata);
int read_ndata64_l(FILE *fin, int bitpos, int ndata, int *idata);
void set_idata(unsigned int *idata, int xmax, int track, int nbit);
void reset_idata(unsigned int *idata, int xmax, int track, int nbit);
int get_idata(unsigned int *idata, int xmax, int track, int nbit);
int read_ndata_r_r(mk5_data_r_t *mk5r, int ndata);
void init_mk5_dat(mk5_data_t *mk5, mk5tok5_info_t *mk5info);
int read_1frame(mk5_data_t *mk5);
int read_1frame_np(mk5_data_t *mk5);
int read_1frame_r(mk5_data_t *mk5);
int read_1frame_m_p(mk5_data_t *mk5);
int read_1frame_l_np(mk5_data_t *mk5);
int read_1frame_r_r(mk5_data_r_t *mk5r); // no more used any where
int fast_atamadashi(mk5_data_t *mk5, int soffset);
int fast_atamadashi_np(mk5_data_t *mk5, int soffset); //non parity version
int fast_atamadashi_r(mk5_data_t *mk5, int soffset);
int fast_atamadashi_r_r(mk5_data_r_t *mk5r, int soffset);
int sync_detect(FILE *fin, int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko);
int sync_detect2(FILE *fin, int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl);
int sync_detect2s(FILE *fin, short int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl);  //nonparity data and 16bit-word version
int sync_detect2new(FILE *fin, int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl,
                 int bitposmax, int parity, int nrzm);  // ver 2003-07-12
int sync_detect2new64(FILE *fin, unsigned int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl,
                 int bitposmax, int parity, int nrzm);  // ver 2003-07-12
void headbit2int4(int *headbit, int *headint4, int parity);
void readM5_8byte(FILE *fin, unsigned int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *ko, int bitposmax, int parity, int nrzm, 
                 int *data1st);
int sync_detect2new64r(FILE *fin, unsigned int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl, int *bitsframe,
                 int bitposmax, int parity, int nrzm, int *data1st);
int sync_detect_full(FILE *fin, int ibit, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl,
                 int bitposmax, int parity, int nrzm);
void mk5_atamadashi_old(FILE *fin, int *ibuf, int n, int ibit, int *ireadr, 
                    int *iposr, int *itime, int *header, int *ko);
int mk5_atamadashi(mk5_read_t *mk5, int soffset);
void sync_detect4_old(FILE *fin, int *ibuf, int n, int nch, int *ibit,
                  int *kold, int *ireadr, 
                 int *ipos, int *itime, int *header, int *head);
void sync_detect4(FILE *fin, unsigned int *ibuf, int n, int nch, int *ibit,
                 int *kold, int *ireadr, 
                 int *ipos, int *itime, int *header, int *head);
void sync_detect4s(FILE *fin, short int *ibuf, int n, int nch, int *ibit,
                  int *kold, int *ireadr, 
                 int *ipos, int *itime, int *header, int *head);
void mk5_atamadashi4(FILE *fin, int *ibuf, int n, int nch, int *ibit, int *kold, int *ireadr, 
                    int *iposr, int *itime, int *header, int * head);
void mk5_atamadashi4s(FILE *fin, short int *ibuf, int n, int nch, int *ibit, int *kold, int *ireadr, 
                    int *iposr, int *itime, int *header, int * head);
void mk5hdbit2int(mk5_read_t *mk5);
void mk5_sync_detect(mk5_read_t *mk5);
void mk5_dat_anal_old(char *mk5name, int *validtrk, int *trktable, 
                  int *header, double *fsampl, int *istart);
void mk5_dat_anal_np(char *mk5name, int *validtrk, int *trktable, 
                  int *header, double *fsampl, int *istart, int *ntrack);
int mk5_dat_anal(char *mk5name, int *validtrk, int *trktable, int *hstack_tbl,
                  int *header, double *fsampl, int *istart, int *ntrack,
                  int *bitsframe, int parity, int nrzm, int mode,
                  int *vcntable);
void mk5_dat_anal_np_full(char *mk5name, int *validtrk, int *trktable, 
                  int *header, double *fsampl, int *istart);  //just for debug purpose
void decodepatn(int *iwork);  // no more use
void parse_hstr(char *hstr, int *hs, int *tr);
void mk5tok5_info_setup(char *info_file, mk5tok5_info_t *mk5info);
void k5tom5_info_setup(char *info_file, mk5tok5_info_t *mk5info);
void make_mk5tok5_info_file_old(char *info_file, char* mk5file, char* vex_file); // interactive operation
void make_mk5tok5_info_file_np(char *info_file, char* mk5file, char* vex_file, 
                               char *sid, int scan, char* program_version,
                               char* dflt_pass);
void make_mk5tok5_info_file(char *info_file, char* mk5file, char* vex_file, 
                            char *sid, int scan, char* program_version,
                            char* dflt_pass);
void make_k5tom5_info_file(char *info_file, char* vex_file, 
                           char *sid, int scan, char* program_version,
                            char* dflt_pass);
void get_bit_track(mk5tok5_info_t *mk5info, int numch, int *ch_table, 
                   int *bit_trk);
void parse_dir_and_name(char *fpath, char *dir, char *fname);
void k5name_make(char *mk5name, char *outdir, int channel, int group, 
                 char *k5name);
void monit_mk5info(mk5tok5_info_t *mk5info);
int Grinfo_fm_Mk5info(mk5tok5_info_t *mk5info, gr_info_t *gr);
void monit_bitpos_info(mk5tok5_info_t *mk5info, gr_info_t *gr);
int head180_to_time(mk5_read_t *mk5, int tr, int *itime);
int conv2k5(mk5tok5_info_t *mk5info, gr_info_t *gr, 
            m5k5runp_t *runp);
int conv2k5_32_l(mk5tok5_info_t *mk5info, gr_info_t *gr, 
            m5k5runp_t *runp);
int conv2k5_32_m(mk5tok5_info_t *mk5info, gr_info_t *gr, 
            m5k5runp_t *runp);
int conv2k5_16_l(mk5tok5_info_t *mk5info, gr_info_t *gr, 
            m5k5runp_t *runp);
int conv2k5_64_l(mk5tok5_info_t *mk5info, gr_info_t *gr, 
            m5k5runp_t *runp);
void shift_bit_trk_table(int *bit_trk, int tot_track);
int ut2mjd(double *d, int year, int month, int day,
           int hour, int min, int sec);
void mjd2ut(int *year, int *month, int *day,
		   int *hour, int *min, int *sec, int j, double d);


#endif
